//===========================================================================
// Createurs : Clement VIGUIER & Jean-Baptiste SILVY
// Fichier : CGShaderManager.cpp
// Description : Classe permettant de gerer des Shaders ecrit en CG.
//===========================================================================


#include <stdio.h>
#include <string.h>
#include "./CGShaderManager.h"


//-------------------------------------------------------------------------
// CONSTRUCTEURS - DESTRUCTEUR --------------------------------------------
//-------------------------------------------------------------------------

CCGShaderManager::CCGShaderManager( void )
{
   m_iNbVertexShaders = m_iNbPixelShaders = 0 ;

   // mise a NULL des tableaux
   for( int i = 0 ; i < NB_MAX_SHADERS ; i++ )
   {
      m_pVertexShaders[i].m_pShader = NULL ;
      m_pVertexShaders[i].m_sName   = NULL ;

      m_pPixelShaders[i].m_pShader = NULL ;
      m_pPixelShaders[i].m_sName   = NULL ;
   }

   // Creation du contexte global
   // TODO : changer ca
   m_oGlobalContext = cgCreateContext() ;
}

//-------------------------------------------------------------------------

CCGShaderManager::~CCGShaderManager( void )
{
   // destruction des vertex shaders
   for( int i = 0 ; i < m_iNbVertexShaders ; i++ )
   {
      delete m_pVertexShaders[i].m_pShader ;
      delete m_pVertexShaders[i].m_sName   ;
   }

   // destruction des pixel shaders
   for( int i = 0 ; i < m_iNbPixelShaders ; i++ )
   {
      delete m_pPixelShaders[i].m_pShader ;
      delete m_pPixelShaders[i].m_sName   ;
   }

   // destruction du contexte
   cgDestroyContext( m_oGlobalContext ) ;
}




//-------------------------------------------------------------------------
// CONSTRUCTEUR DE SHADER -------------------------------------------------
//-------------------------------------------------------------------------

CCGVertexShader * CCGShaderManager::createVertexShaderFromSourceFile( const char * filename, const char * entry )
{
   // on verifie que le nombre max de shaders n'est pas atteint
   if( m_iNbVertexShaders == NB_MAX_SHADERS ) { return NULL ; }

   // creation du nouveau vertex shader
   CCGVertexShader * newShader = new CCGVertexShader( m_oGlobalContext, filename, entry, CG_SOURCE, (CGprofile)0, 0 ) ;

   // ajout du shader en interne
   m_pVertexShaders[m_iNbVertexShaders].m_pShader = newShader                      ;
   m_pVertexShaders[m_iNbVertexShaders].m_sName   = new char[strlen(filename) + 1] ;
   strcpy( m_pVertexShaders[m_iNbVertexShaders].m_sName, filename ) ;
   
   m_iNbVertexShaders++ ;

   return newShader ;
}

//-------------------------------------------------------------------------

CCGPixelShader * CCGShaderManager::createPixelShaderFromSourceFile( const char * filename, const char * entry )
{
   // on verifie que le nombre max de shaders n'est pas atteint
   if( m_iNbPixelShaders == NB_MAX_SHADERS ) { return NULL ; }

   // creation du nouveau vertex shader
   CCGPixelShader * newShader = new CCGPixelShader( m_oGlobalContext, filename, entry, CG_SOURCE, (CGprofile)0, 0 ) ;

   // ajout du shader en interne
   m_pPixelShaders[m_iNbPixelShaders].m_pShader = newShader                      ;
   m_pPixelShaders[m_iNbPixelShaders].m_sName   = new char[strlen(filename) + 1] ;
   strcpy( m_pPixelShaders[m_iNbPixelShaders].m_sName, filename ) ;

   m_iNbPixelShaders++ ;

   return newShader ;
}

//-------------------------------------------------------------------------

bool CCGShaderManager::manageExistingVertexShader( CCGVertexShader * shader, const char * filename )
{
   // on verifie que le nombre max de shaders n'est pas atteint
   if( m_iNbVertexShaders == NB_MAX_SHADERS ) { return false ; }

   // ajout du shader en interne
   m_pVertexShaders[m_iNbVertexShaders].m_pShader = shader                         ;
   m_pVertexShaders[m_iNbVertexShaders].m_sName   = new char[strlen(filename) + 1] ;
   strcpy( m_pVertexShaders[m_iNbVertexShaders].m_sName, filename ) ;

   m_iNbVertexShaders++ ;

   return true ;
}

//-------------------------------------------------------------------------

bool CCGShaderManager::manageExistingPixelShader(  CCGPixelShader  * shader, const char * filename )
{
   // on verifie que le nombre max de shaders n'est pas atteint
   if( m_iNbPixelShaders == NB_MAX_SHADERS ) { return false ; }

   // ajout du shader en interne
   m_pPixelShaders[m_iNbPixelShaders].m_pShader = shader                         ;
   m_pPixelShaders[m_iNbPixelShaders].m_sName   = new char[strlen(filename) + 1] ;
   strcpy( m_pPixelShaders[m_iNbPixelShaders].m_sName, filename ) ;

   m_iNbPixelShaders++ ;

   return true ;
}




//-------------------------------------------------------------------------
// ACCESSEURS -------------------------------------------------------------
//-------------------------------------------------------------------------

CCGVertexShader * CCGShaderManager::getVertexShader( const char * filename ) const
{
   for( int i = 0 ; i < m_iNbVertexShaders ; i++ )
   {
      if( strcmp( filename, m_pVertexShaders[i].m_sName ) == 0 )
      {
         return (CCGVertexShader *)m_pVertexShaders[i].m_pShader ;
      }
   }

   return NULL ;
}

//-------------------------------------------------------------------------

CCGPixelShader * CCGShaderManager::getPixelShader( const char * filename ) const
{
   for( int i = 0 ; i < m_iNbPixelShaders ; i++ )
   {
      if( strcmp( filename, m_pPixelShaders[i].m_sName ) == 0 )
      {
         return (CCGPixelShader *)m_pPixelShaders[i].m_pShader ;
      }
   }

   return NULL ;
}

//-------------------------------------------------------------------------

CCGShader * CCGShaderManager::getShader( EShaderType type, const char * filename ) const
{
   return type == CGSM_PIXEL_SHADER ?
      (CCGShader *)getPixelShader(  filename ) :
      (CCGShader *)getVertexShader( filename )  ;
}





//-------------------------------------------------------------------------
// MODIFIEURS -------------------------------------------------------------
//-------------------------------------------------------------------------

void CCGShaderManager::recompileAll( EShaderType type )
{
   // on regarde s'il faut compiler les vertex shaders
   if( type & CGSM_VERTEX_SHADER )
   {
      for( int i = 0 ; i < m_iNbVertexShaders ; i++ )
      {
         m_pVertexShaders[i].m_pShader->recompile() ;
      }
   }

   // on regarde s'il faut compiler les pixel shaders
   if( type & CGSM_PIXEL_SHADER )
   {
      for( int i = 0 ; i < m_iNbPixelShaders ; i++ )
      {
         m_pPixelShaders[i].m_pShader->recompile() ;
      }
   }
}

//-------------------------------------------------------------------------

void CCGShaderManager::recompileVertexShader( const char * filename )
{
   CCGShader * found = getVertexShader( filename ) ;
   if( found != NULL ) { found->recompile() ; }
}

//-------------------------------------------------------------------------

void CCGShaderManager::recompilePixelShader( const char * filename )
{
   CCGShader * found = getPixelShader( filename ) ;
   if( found != NULL ) { found->recompile() ; }
}

//-------------------------------------------------------------------------

void CCGShaderManager::recompileShader( EShaderType type, const char * filename )
{
   CCGShader * found = getShader( type, filename ) ;
   if( found != NULL ) { found->recompile() ; }
}

//-------------------------------------------------------------------------
